use tokio::task;

use crate::tcp_relay_client::client_config::ClientConfig;
use crate::tcp_relay_server::server_config::ServerConfig;
use crate::test::dummy_tcp_sockets::{
    dummy_tcp_client_square_root_requests, dummy_tcp_client_write_then_read_conn, wait_for_port,
    DummyTCPServer,
};
use crate::test::{get_port_number, PortsAllocation, LOCALHOST};

const VALID_TOKEN: &str = "AvalidTOKEN";

const DATA_1: &[u8] = "DATA1".as_bytes();
const DATA_2: &[u8] = "DATA2".as_bytes();

fn port(index: u16) -> u16 {
    get_port_number(PortsAllocation::ValidWithTokenAuth, index)
}

#[tokio::test(flavor = "multi_thread", worker_threads = 5)]
async fn valid_with_token_auth() {
    let _ = env_logger::builder().is_test(true).try_init();

    tokio::spawn(async move {
        // Start internal service
        let local_server = DummyTCPServer::start(port(1)).await;
        local_server.next_conn_square_operations().await;
    });

    let local_set = task::LocalSet::new();
    local_set
        .run_until(async move {
            wait_for_port(port(1)).await;

            // Start server relay
            task::spawn_local(crate::tcp_relay_server::run_app(ServerConfig {
                tokens: vec![VALID_TOKEN.to_string()],
                tokens_file: None,
                ports: vec![port(1)],
                upstream_server: "127.0.0.1".to_string(),
                listen_address: format!("127.0.0.1:{}", port(0)),
                increment_ports: 1,
                tls_cert: None,
                tls_key: None,
                tls_client_auth_root_cert: None,
                tls_revocation_list: None,
            }));
            wait_for_port(port(0)).await;

            // Start client relay
            task::spawn(crate::tcp_relay_client::run_app(ClientConfig {
                token: Some(VALID_TOKEN.to_string()),
                relay_url: format!("http://{}:{}", LOCALHOST, port(0)),
                listen_address: LOCALHOST.to_string(),
                root_certificate: None,
                ..Default::default()
            }));
            wait_for_port(port(2)).await;
        })
        .await;
}
